
from twisted.spread import pb
from twisted.spread.util import Pager

from twisted.internet import defer

import os

import acl
import session

class FileSession(session.ServerSession, session.FileSSM):
    pass

# A Perspective providing Filetransfer Sessions
class FilePerspective(pb.Perspective):
    def perspective_getsession(self):
        return FileSession()

class FilePager(Pager):
    def __init__(self, collector, file, callback=None, *args, **kws):
        self.file = file
        self.chunkSize = 8192
        Pager.__init__(self, collector, callback, *args, **kws)
    def nextPage(self):
        data = self.file.read(self.chunkSize)
        if len(data) < self.chunkSize:
            self.stopPaging()
        return data

class AbstractPagerizer(pb.Referenceable):
    def __init__(self, *args, **kws):
        pass
    def remote_getPages(self, collector):
        pass

class FilePagerizer(AbstractPagerizer):
    def __init__(self, filename, callback=None, *args, **kws):
        self.filename = filename
        self.file = file(filename)
        self.callback = callback
        self.args = args
        self.kws = kws
    def remote_getPages(self, collector):
        FilePager(collector, self.file, self.callback, *self.args, **self.kws)

class AbstractCollector(pb.Referenceable):
    def __init__(self, callback=None, *args, **kws):
        self.setcallback(callback, *args, **kws)
    def remote_gotPage(self, page):
        self.gotPage(page)
    def gotPage(self, page):
        pass
    def remote_endedPaging(self):
        self.endedPaging()
        if self.callback is not None:
            self.callback(*self.args, **self.kws)
    def endedPaging(self):
        pass
    def setcallback(self, callback, *args, **kws):
        self.callback = callback
        self.args = args
        self.kws = kws

class PrintCollector(AbstractCollector):
    def __init__(self, *args, **kws):
        apply(AbstractCollector.__init__, [self] + list(args), kws)
    def gotPage(self, page):
        print page

class FileCollector(AbstractCollector):
    def __init__(self, file, *args, **kws):
        apply(AbstractCollector.__init__, [self] + list(args), kws)
        self.file = file
    def gotPage(self, page):
        self.file.write(page)

# Page helper (both up- and download)
# Get pages from Remote Pager and give them to the collector.
def pages_collect(pagerizer, collector):
    pagerizer.callRemote("getPages", collector)

# Download
def getfile(session, filename, collector):
    defr = defer.Deferred()
    collector.setcallback(defr.callback, session)
    tmp = session.callRemote("pagerize", filename)
    tmp.addCallback(pages_collect, collector)
    return defr

# Upload
def sendfile(session, local, remote):
    defr = defer.Deferred()
    pagerizer = FilePagerizer(local, defr.callback, session)
    session.callRemote('receivefile', pagerizer, remote)
    return defr

